/*---------------------------------------------------------------------------*/
//       Author : hiyohiyo
//         Mail : hiyohiyo@crystalmark.info
//          Web : http://crystalmark.info/
//      License : The modified BSD license
//
//                                Copyright 2007 hiyohiyo, All rights reserved.
/*---------------------------------------------------------------------------*/

#include "stdafx.h"
#include "CrystalCpuid.h"
#include "CrystalCpuidDlg.h"

#include "GetFileVersion.h"
#include "IsCurrentUserLocalAdministrator.h"

#include <mmsystem.h>
#pragma comment(lib, "winmm.lib")

#ifdef RUN_TIME_DYNAMIC_LINKING 
#include "../OpenLibSys/OlsApiInit.h"
#else // for Load-Time Dynamic Linking
#include "../OpenLibSys/OlsApi.h"
#ifdef _M_X64
#pragma comment(lib, "../../Estel/OpenLibSysX64.lib")
#else if
#pragma comment(lib, "../../Estel/OpenLibSys.lib")
#endif
#endif

#ifdef _DEBUG
#define new DEBUG_NEW
#endif

// CCrystalCpuidApp

BEGIN_MESSAGE_MAP(CCrystalCpuidApp, CWinApp)
	ON_COMMAND(ID_HELP, &CWinApp::OnHelp)
END_MESSAGE_MAP()

//-----------------------------------------------------------------------------
// Prototypes
//-----------------------------------------------------------------------------
static BOOL IsFileExistEx(const TCHAR* path, const TCHAR* fileName);
static BOOL RunAsRestart();

// CCrystalCpuidApp construction

CCrystalCpuidApp::CCrystalCpuidApp()
{
}

CCrystalCpuidApp::~CCrystalCpuidApp()
{
	if(m_WmiInfo != NULL)
	{
		delete m_WmiInfo;
		m_WmiInfo = NULL;
	}
}

// The one and only CCrystalCpuidApp object

CCrystalCpuidApp theApp;


// CCrystalCpuidApp initialization

BOOL CCrystalCpuidApp::InitInstance()
{
	INITCOMMONCONTROLSEX InitCtrls;
	InitCtrls.dwSize = sizeof(InitCtrls);
	InitCtrls.dwICC = ICC_WIN95_CLASSES;
	InitCommonControlsEx(&InitCtrls);
	CWinApp::InitInstance();

	// IE Version Check.
	if(GetFileVersion(_T("Shdocvw.dll")) < 471)
	{
		AfxMessageBox(_T("CrystalCPUID is required IE6 or later."));
		return FALSE;
	}

	m_WmiInfo = NULL;
	m_OsVer.dwOSVersionInfoSize = sizeof(OSVERSIONINFO);
	GetVersionEx(&m_OsVer);
	if(m_OsVer.dwPlatformId == VER_PLATFORM_WIN32_NT)
	{
		m_IsNT = TRUE;
	}
	else
	{
		m_IsNT = FALSE;
	}

	CString DefaultTheme;
	CString DefaultLanguage;
	TCHAR tmp[MAX_PATH];
	TCHAR *ptrEnd;

	GetModuleFileName(NULL, tmp, MAX_PATH);
	if((ptrEnd = _tcsrchr(tmp, '\\')) != NULL){*ptrEnd = '\0';}

	m_ThemeDir.Format(_T("%s\\%s"), tmp, THEME_DIR);
	m_LangDir.Format(_T("%s\\%s"), tmp, LANGUAGE_DIR);

	DefaultTheme.Format(_T("%s\\%s"), tmp, DEFAULT_THEME);
	DefaultLanguage.Format(_T("%s\\%s"), tmp, DEFAULT_LANGUAGE);

	m_MainDlgPath.Format(_T("%s\\") DIALOG_DIR MAIN_DIALOG, tmp);
	m_AboutDlgPath.Format(_T("%s\\") DIALOG_DIR ABOUT_DIALOG, tmp);
	m_FeatureFlagsDlgPath.Format(_T("%s\\")DIALOG_DIR FEATURE_FLAGS_DIALOG, tmp);
	m_BiosInfoDlgPath.Format(_T("%s\\")DIALOG_DIR BIOS_INFO_DIALOG, tmp);
	m_CpuidInfoDlgPath.Format(_T("%s\\")DIALOG_DIR CPUID_INFO_DIALOG, tmp);
	m_SystemInfoDlgPath.Format(_T("%s\\")DIALOG_DIR SYSTEM_INFO_DIALOG, tmp);

	if(! IsFileExistEx(m_MainDlgPath, MAIN_DIALOG))					{	return FALSE;	}
	if(! IsFileExistEx(m_AboutDlgPath, ABOUT_DIALOG))				{	return FALSE;	}
	if(! IsFileExistEx(m_FeatureFlagsDlgPath, FEATURE_FLAGS_DIALOG)){	return FALSE;	}
	if(! IsFileExistEx(m_BiosInfoDlgPath, BIOS_INFO_DIALOG))		{	return FALSE;	}
	if(! IsFileExistEx(m_CpuidInfoDlgPath, CPUID_INFO_DIALOG))		{	return FALSE;	}
	if(! IsFileExistEx(m_SystemInfoDlgPath, SYSTEM_INFO_DIALOG))	{	return FALSE;	}
//	if(! IsFileExistEx(m_Path, _DIALOG))							{	return FALSE;	}

	if(! IsFileExistEx(DefaultTheme, DEFAULT_THEME))				{	return FALSE;	}
	if(! IsFileExistEx(DefaultLanguage, DEFAULT_LANGUAGE))			{	return FALSE;	}

#ifdef RUN_TIME_DYNAMIC_LINKING
	m_hOpenLibSys = NULL;
	if(InitOpenLibSys(&m_hOpenLibSys) != TRUE)
	{
		AfxMessageBox(_T("DLL Load Error!!"));
		return FALSE;
	}
#else if
	InitializeDll();
#endif

// for Windows NT family
#ifdef _UNICODE
	if(! IsCurrentUserLocalAdministrator()
	&& GetDriverStatus() != OLS_DRIVER_STATUS_AUTO_START)
	{
		if(m_OsVer.dwMajorVersion < 6)
		{
			AfxMessageBox(_T("CrystalCPUID ̎sɂ͊ǗҌKvłB"));
		}
		RunAsRestart();
		return FALSE;
	}
#endif

	switch(GetDllStatus())
	{
	case OLS_DLL_NO_ERROR:
		break;
	case OLS_DLL_UNSUPPORTED_PLATFORM:
		AfxMessageBox(_T("DLL Status Error!! UNSUPPORTED_PLATFORM"));
		return FALSE;
		break;
	case OLS_DLL_DRIVER_NOT_LOADED:
		AfxMessageBox(_T("DLL Status Error!! DRIVER_NOT_LOADED"));
		return FALSE;
		break;
	case OLS_DLL_DRIVER_NOT_FOUND:
		AfxMessageBox(_T("DLL Status Error!! DRIVER_NOT_FOUND"));
		return FALSE;
		break;
	case OLS_DLL_DRIVER_UNLOADED:
		AfxMessageBox(_T("DLL Status Error!! DRIVER_UNLOADED"));
		return FALSE;
		break;
	case OLS_DLL_DRIVER_NOT_LOADED_ON_NETWORK:
		AfxMessageBox(_T("DLL Status Error!! DRIVER_NOT_LOADED_ON_NETWORK"));
		return FALSE;
		break;
	case OLS_DLL_UNKNOWN_ERROR:
	default:
		AfxMessageBox(_T("DLL Status Error!! UNKNOWN_ERROR"));
		return FALSE;
		break;
	}

	// Multimedia Timer Setting
	TIMECAPS tc;
	timeGetDevCaps(&tc,sizeof(TIMECAPS));
	timeBeginPeriod(tc.wPeriodMin);
	
	CCrystalCpuidDlg dlg;
	m_pMainWnd = &dlg;
	INT_PTR nResponse = dlg.DoModal();

	timeEndPeriod(tc.wPeriodMin);

#ifdef RUN_TIME_DYNAMIC_LINKING
	DeinitOpenLibSys(&m_hOpenLibSys);
#else if
	DeinitializeDll();
#endif

	return FALSE;
}

CWmiInfo* CCrystalCpuidApp::GetWmiInfo()
{
	if(m_WmiInfo == NULL)
	{
		m_WmiInfo = new CWmiInfo;
	}

	return m_WmiInfo;
}

BOOL IsFileExistEx(const TCHAR* path, const TCHAR* fileName)
{
	if(! IsFileExist(path))
	{
		CString cstr;
		cstr.Format(_T("Not Found \"%s\"."), fileName); 
		AfxMessageBox(cstr);
		return FALSE;
	}
	return TRUE;
}

BOOL RunAsRestart()
{
	int count;
#ifdef _UNICODE
	TCHAR** cmd = ::CommandLineToArgvW(::GetCommandLine(), &count);
#else
	TCHAR** cmd = ::__argv;
	count = ::__argc;
#endif

	if(count < 2 || _tcscmp(cmd[1], _T("runas")) != 0)
	{
		TCHAR path[MAX_PATH];
		::GetModuleFileName(NULL, path, MAX_PATH);
		if(::ShellExecute(NULL, _T("runas"), path, _T("runas"), NULL, SW_SHOWNORMAL)
			> (HINSTANCE)32)
		{
			return TRUE;
		}
	}
	return FALSE;
}
