//-----------------------------------------------------------------------------
//     Author : hiyohiyo
//       Mail : hiyohiyo@crystalmark.info
//        Web : http://openlibsys.org/
//    License : The modified BSD license
//
//                     Copyright 2007-2008 OpenLibSys.org. All rights reserved.
//-----------------------------------------------------------------------------

#include "stdafx.h"
#include "OlsApi.h"
#include "DmiInfo.h"
#include "ItemId.h"

#include <stdio.h>

#pragma warning(disable:4996)

CDmiInfo::CDmiInfo()
{
	InitData();
}

CDmiInfo::~CDmiInfo()
{

}

DWORD CDmiInfo::GetString(DWORD id, PCHAR s /* str */, DWORD z /* size */)
{
	errno_t e;

	switch(id)
	{
	case	DMI_SM_BIOS_VERSION:	e = strcpy_s(s, z, DmiSmBiosVersion);		break;
	case	DMI_BIOS_VENDOR:		e = strcpy_s(s, z, DmiBiosVendor);			break;
	case	DMI_BIOS_VERSION:		e = strcpy_s(s, z, DmiBiosVersion);			break;
	case	DMI_BIOS_RELEASE_DATE:	e = strcpy_s(s, z, DmiBiosReleaseDate);		break;
	case	DMI_BIOS_ROM_SIZE:		e = strcpy_s(s, z, DmiBiosRomSize);			break;
	case	DMI_MOTHER_MANUFACTURER:e = strcpy_s(s, z, DmiMotherManufacturer);	break;
	case	DMI_MOTHER_PRODUCT:		e = strcpy_s(s, z, DmiMotherProduct);		break;
	case	DMI_MOTHER_VERSION:		e = strcpy_s(s, z, DmiMotherVersion);		break;
	case	DMI_CPU_SOCKET:			e = strcpy_s(s, z, DmiCpuSocket);			break;
	case	DMI_CPU_MANUFACTURER:	e = strcpy_s(s, z, DmiCpuManufacturer);		break;
	case	DMI_CPU_VERSION:		e = strcpy_s(s, z, DmiCpuVersion);			break;
	case	DMI_CPU_CURRENT_CLOCK:	e = strcpy_s(s, z, DmiCpuCurrentClock);		break;
	case	DMI_CPU_EXTERNAL_CLOCK:	e = strcpy_s(s, z, DmiCpuExternalClock);	break;
	case	DMI_CPU_MAX_CLOCK:		e = strcpy_s(s, z, DmiCpuMaxClock);			break;
	default:
		strcpy_s(s, z, "");
		return -1;
		break;
	}
	return e;
}

DWORD CDmiInfo::GetData(DWORD id, PDWORD v)
{
	switch(id)
	{
	case	DMI_STATUS:				*v = DmiStatus;						break;
	default:
		*v = -1;
		return -1;
	}
	return 0;
}

#define MAKEDWORD(a,b,c,d)	((d<<24)+(c<<16)+(b<<8)+(a))

static BOOL CheckSum(const BYTE *buf, int length)
{
	BYTE sum = 0;
	for(int i=0; i < length; i++)
	{
		sum += buf[i];
	}
	return (sum==0);
}

static VOID DmiString(PCHAR str, DmiHeader* dmi, UCHAR id)
{
	PCHAR p = (PCHAR)dmi;
	p += dmi->Length;
	while(id > 1 && *p)
	{
		p += strlen(p);
		p++;
		id--;
	}
	// ASCII Filter
	int len = (int)strlen(p);
	for(int i = 0; i < len; i++)
	{
		if(p[i] < 32 || p[i] == 127)
		{
			p[i]='.';
		}
	}
	strcpy(str, p);
}

BOOL CDmiInfo::InitData()
{
	DmiStatus = FALSE;
	DmiSmBiosVersion[0] = '\0';
	DmiBiosVendor[0] = '\0';
	DmiBiosVersion[0] = '\0';
	DmiBiosReleaseDate[0] = '\0';
	DmiBiosRomSize[0] = '\0';
	DmiMotherManufacturer[0] = '\0';
	DmiMotherProduct[0] = '\0';
	DmiMotherVersion[0] = '\0';
	DmiCpuSocket[0] = '\0';
	DmiCpuManufacturer[0] = '\0';
	DmiCpuVersion[0] = '\0';
	DmiCpuCurrentClock[0] = '\0';
	DmiCpuExternalClock[0] = '\0';
	DmiCpuMaxClock[0] = '\0';

	UCHAR b[MAP_MEMORY_SIZE];
	UCHAR *d = NULL;
	ReadPhysicalMemory(0x000F0000, b, MAP_MEMORY_SIZE, 1);

	int j;
	UCHAR *p;
	UCHAR *next;
	DWORD EntryPoint;
	DWORD StructureLength;
	DWORD NumberOfStructures;
	BOOL Flag = FALSE;
	p = (UCHAR *)b;

	for(j = 0; j< MAP_MEMORY_SIZE; j += 16)
	{
		if(memcmp(p, "_SM_", 4) == 0)
		{
			sprintf(DmiSmBiosVersion, "%d.%d", p[6], p[7]);
		}
		if(memcmp(p, "_DMI_", 5) == 0)
		{
			EntryPoint = MAKEDWORD(p[8], p[9], p[0xA], p[0xB]);
			StructureLength = MAKEWORD(p[6], p[7]);
			NumberOfStructures = MAKEWORD(p[0xC], p[0xD]);
			if(CheckSum(p, 0xF))
			{
				DmiStatus = TRUE;
				Flag = TRUE;
			}
			break;
		}
		p+=16;
	}

	if(Flag == FALSE)
	{
		return FALSE;
	}

	d = new UCHAR[StructureLength];
	ReadPhysicalMemory(EntryPoint, d, StructureLength, 1);

	DWORD num;
	p = d;
	DmiHeader* dmi;
	for(num = 0;num < NumberOfStructures; num++)
	{
		dmi = (DmiHeader*)p;
		switch(dmi->Type)
		{
///////////////////////////////////////////////////////////////////////////////////////////////////
// 00 BIOS Information
///////////////////////////////////////////////////////////////////////////////////////////////////
		case 0:
			DmiString(DmiBiosVendor, dmi, p[4]);
			DmiString(DmiBiosVersion, dmi, p[5]);
			DmiString(DmiBiosReleaseDate, dmi, p[8]);
			sprintf(DmiBiosRomSize, "%d KB", ((p[9] + 1) << 16) / 1024);
			break;
///////////////////////////////////////////////////////////////////////////////////////////////////
// 02 Base Board Information
///////////////////////////////////////////////////////////////////////////////////////////////////
		case 2:
			DmiString(DmiMotherManufacturer, dmi, p[4]);
			DmiString(DmiMotherProduct, dmi, p[5]);
			DmiString(DmiMotherVersion, dmi, p[6]);
			break;
///////////////////////////////////////////////////////////////////////////////////////////////////
// 04 Processor Information
///////////////////////////////////////////////////////////////////////////////////////////////////
		case 4:
			DmiString(DmiCpuSocket, dmi, p[4]);
			DmiString(DmiCpuManufacturer, dmi, p[7]);
			DmiString(DmiCpuVersion, dmi, p[0x10]);
			sprintf(DmiCpuExternalClock, "%d MHz", MAKEWORD(p[0x12], p[0x13]));
			sprintf(DmiCpuMaxClock, "%d MHz", MAKEWORD(p[0x14], p[0x15]));
			sprintf(DmiCpuCurrentClock, "%d MHz", MAKEWORD(p[0x16], p[0x17]));
			break;
			}
		// next 
		next = p + dmi->Length;
		while(next[0] !=0 || next[1] != 0)
		{
			next++;
		}
		p = next + 2;
	}
	delete d;

	return TRUE;
}