//-----------------------------------------------------------------------------
//     Author : hiyohiyo
//       Mail : hiyohiyo@crystalmark.info
//        Web : http://openlibsys.org/
//    License : The modified BSD license
//
//                          Copyright 2007 OpenLibSys.org. All rights reserved.
//-----------------------------------------------------------------------------

#include "stdafx.h"
#include <tchar.h>
#include <winioctl.h>
#include <stddef.h>
#include <malloc.h>
#include <intrin.h>
#include "OlsDll.h"
#include "OlsApi.h"
#include "OlsDef.h"
#include "Driver.h"
#include "OlsIoctl.h"

//-----------------------------------------------------------------------------
//
// Defines
//
//-----------------------------------------------------------------------------

const BYTE gPciNumberOfDevice =	32;
const BYTE gPciNumberOfFunction = 8;

//-----------------------------------------------------------------------------
//
// Global
//
//-----------------------------------------------------------------------------

BYTE gPciMaxNumberOfBus = 255;

extern HANDLE gHandle;

extern BOOL gIsNT;
extern BOOL gIsCpuid;
extern BOOL gIsMsr;
extern BOOL gIsTsc;
extern BOOL gInitDll;

extern DWORD gDllStatus;
extern DWORD gDriverType;
extern TCHAR gDriverPath[MAX_PATH];

#ifdef _OPEN_LIB_SYS
extern DWORD gDriverStatus;
#endif

//-----------------------------------------------------------------------------
//
// DLL
//
//-----------------------------------------------------------------------------

DWORD WINAPI GetDllStatus()
{
	if(gDllStatus == OLS_DLL_NO_ERROR
	&& gDriverType >= OLS_DRIVER_TYPE_WIN_NT
	&& GetDriverVersion(NULL, NULL, NULL, NULL) == 0
	)
	{
		gDllStatus = OLS_DLL_DRIVER_UNLOADED;
	}

	return gDllStatus;
}

DWORD WINAPI GetDllVersion(PBYTE major, PBYTE minor, PBYTE revision, PBYTE release)
{
	if(major != NULL && minor != NULL && revision != NULL && release != NULL)
	{
		*major = OLS_MAJOR_VERSION;
		*minor = OLS_MINOR_VERSION;
		*revision = OLS_REVISION;
		*release = OLS_RELESE;
	}

	return OLS_VERSION;
}

DWORD WINAPI GetDriverVersion(PBYTE major, PBYTE minor, PBYTE revision, PBYTE release)
{
	if(gHandle == INVALID_HANDLE_VALUE){return 0;}

	DWORD	returnedLength = 0;
	BOOL	result = FALSE;
	DWORD	value = 0;

	result = DeviceIoControl(
                        gHandle,
                        IOCTL_OLS_GET_DRIVER_VERSION,
                        NULL,     
                        0,
                        &value,
                        sizeof(value),
                        &returnedLength,
                        NULL
                        );

	if(major != NULL && minor != NULL && revision != NULL && release != NULL)
	{
		*major = (BYTE)((value >> 24) & 0xFF);
		*minor = (BYTE)((value >> 16) & 0xFF);
		*revision = (BYTE)((value >> 8) & 0xFF);
		*release = (BYTE)((value) & 0xFF);		
	}
	
	return value;
}

DWORD WINAPI GetDriverType()
{
	return gDriverType;
}

#ifdef _OPEN_LIB_SYS

BOOL WINAPI InstallOpenLibSys()
{
	if(gInitDll == FALSE){InitializeDll();}

	if(gIsNT)
	{
		return ManageDriver(OLS_DRIVER_ID, gDriverPath, OLS_DRIVER_SYSTEM_INSTALL);
	}
	else
	{
		return FALSE;
	}
}

BOOL WINAPI UninstallOpenLibSys()
{
	if(gInitDll == FALSE){InitializeDll();}

	if(gIsNT)
	{
		return ManageDriver(OLS_DRIVER_ID, gDriverPath, OLS_DRIVER_SYSTEM_UNINSTALL);
	}
	else
	{
		return FALSE;
	}
}

DWORD WINAPI GetDriverStatus()
{
	if(gDllStatus == OLS_DLL_NO_ERROR
	&& gDriverType >= OLS_DRIVER_TYPE_WIN_NT
	&& GetDriverVersion(NULL, NULL, NULL, NULL) == 0
	)
	{
		gDriverStatus = OLS_DRIVER_STATUS_UNKNOWN_ERROR;
	}

	return gDriverStatus;
}

#endif

//-----------------------------------------------------------------------------
//
// CPU
//
//-----------------------------------------------------------------------------

BOOL WINAPI IsCpuid()
{
	return gIsCpuid;
}

BOOL WINAPI IsMsr()
{
	return gIsMsr;
}

BOOL WINAPI IsTsc()
{
	return gIsTsc;
}

BOOL WINAPI Rdmsr(DWORD index, PDWORD eax, PDWORD edx)
{
	if(gHandle == INVALID_HANDLE_VALUE){return FALSE;}

	if(eax == NULL || edx == NULL || gIsMsr == FALSE)
	{
		return FALSE;
	}

	DWORD	returnedLength = 0;
 	BOOL	result = FALSE;
	BYTE	outBuf[8] = {0};

	result = DeviceIoControl(
					gHandle,
					IOCTL_OLS_READ_MSR,
					&index,
					sizeof(index),
					&outBuf,
					sizeof(outBuf),
					&returnedLength,
					NULL
					);

	memcpy(eax, outBuf, 4);
	memcpy(edx, outBuf + 4, 4);

	return result;
}

BOOL WINAPI RdmsrEx(DWORD index, PDWORD eax, PDWORD edx, DWORD_PTR affinityMask)
{
	BOOL		result = FALSE;
	DWORD_PTR	mask = 0;

	if(gIsNT)
	{
		mask = SetThreadAffinityMask(GetCurrentThread(), affinityMask);
	}

	result = Rdmsr(index, eax, edx);

	if(gIsNT && mask)
	{
		SetThreadAffinityMask(GetCurrentThread(), mask);
	}

	return result;
}

BOOL WINAPI Wrmsr(DWORD index, DWORD eax, DWORD edx)
{
	if(gHandle == INVALID_HANDLE_VALUE){return FALSE;}

	if(gIsMsr == FALSE)
	{
		return FALSE;
	}

	DWORD	returnedLength = 0;
 	BOOL	result = FALSE;
	DWORD	outBuf;
	OLS_WRITE_MSR_INPUT inBuf;

	inBuf.Register = index;
	inBuf.Value.HighPart = edx;
	inBuf.Value.LowPart = eax;

	result = DeviceIoControl(
					gHandle,
					IOCTL_OLS_WRITE_MSR,
					&inBuf,
					sizeof(inBuf),
					&outBuf,
					sizeof(outBuf),
					&returnedLength,
					NULL
					);

	return result;
}

BOOL WINAPI WrmsrEx(DWORD index, DWORD eax, DWORD edx, DWORD_PTR affinityMask)
{
	BOOL		result = FALSE;
	DWORD_PTR	mask = 0;

	if(gIsNT)
	{
		mask = SetThreadAffinityMask(GetCurrentThread(), affinityMask);
	}

	result = Wrmsr(index, eax, edx);

	if(gIsNT && mask)
	{
		SetThreadAffinityMask(GetCurrentThread(), mask);
	}

	return result;
}

BOOL WINAPI Rdpmc(DWORD index, PDWORD eax, PDWORD edx)
{
	if(gHandle == INVALID_HANDLE_VALUE){return FALSE;}

	if(eax == NULL || edx == NULL)
	{
		return FALSE;
	}

	DWORD	returnedLength = 0;
	BOOL	result = FALSE;
	BYTE	outBuf[8] = {0};

	result = DeviceIoControl(
					gHandle,
					IOCTL_OLS_READ_PMC,
					&index,	
					sizeof(index),
					&outBuf,
					sizeof(outBuf),
					&returnedLength,
					NULL
					);

	memcpy(eax, outBuf, 4);
	memcpy(edx, outBuf + 4, 4);

	return result;
}

BOOL WINAPI RdpmcEx(DWORD index, PDWORD eax, PDWORD edx, DWORD_PTR affinityMask)
{
	BOOL		result = FALSE;
	DWORD_PTR	mask = 0;

	if(gIsNT)
	{
		mask = SetThreadAffinityMask(GetCurrentThread(), affinityMask);
	}

	result = Rdpmc(index, eax, edx);

	if(gIsNT && mask)
	{
		SetThreadAffinityMask(GetCurrentThread(), mask);
	}

	return result;
}

BOOL WINAPI Cpuid(DWORD index, PDWORD eax, PDWORD ebx, PDWORD ecx, PDWORD edx)
{
	if(eax == NULL || ebx == NULL || ecx == NULL || edx == NULL || gIsCpuid == FALSE)
	{
		return FALSE;
	}

	int info[4];

	__cpuid(info, index);
	*eax = info[0];
	*ebx = info[1];
	*ecx = info[2];
	*edx = info[3];

	return TRUE;
}

BOOL WINAPI CpuidEx(DWORD index, PDWORD eax, PDWORD ebx, PDWORD ecx, PDWORD edx, DWORD_PTR affinityMask)
{
	BOOL		result = FALSE;
	DWORD_PTR	mask = 0;

	if(gIsNT)
	{
		mask = SetThreadAffinityMask(GetCurrentThread(), affinityMask);
	}

	result = Cpuid(index, eax, ebx, ecx, edx);

	if(gIsNT && mask)
	{
		SetThreadAffinityMask(GetCurrentThread(), mask);
	}

	return result;
}

BOOL WINAPI Rdtsc(PDWORD eax, PDWORD edx)
{
	if(eax == NULL || edx == NULL || gIsTsc == FALSE)
	{
		return FALSE;
	}

	ULONGLONG value = 0;
	
	value = __rdtsc();
	*eax = (DWORD)((value >>  0) & 0xFFFFFFFF);
	*edx = (DWORD)((value >> 32) & 0xFFFFFFFF);

	return TRUE;
}

BOOL WINAPI RdtscEx(PDWORD eax, PDWORD edx, DWORD_PTR affinityMask)
{
	BOOL		result = FALSE;
	DWORD_PTR	mask = 0;

	if(gIsNT)
	{
		mask = SetThreadAffinityMask(GetCurrentThread(), affinityMask);
	}

	result = Rdtsc(eax, edx);

	if(gIsNT && mask)
	{
		SetThreadAffinityMask(GetCurrentThread(), mask);
	}

	return result;
}

BOOL WINAPI Hlt()
{
	if(gHandle == INVALID_HANDLE_VALUE){return FALSE;}

	DWORD	returnedLength = 0;
 	BOOL	result = FALSE;

	result = DeviceIoControl(
					gHandle,
					IOCTL_OLS_HALT,	
					NULL,
					0,
					NULL,
					0,
					&returnedLength,
					NULL
					);

	return result;
}

BOOL WINAPI HltEx(DWORD_PTR affinityMask)
{
	BOOL		result = FALSE;
	DWORD_PTR	mask = 0;

	if(gIsNT)
	{
		mask = SetThreadAffinityMask(GetCurrentThread(), affinityMask);
	}

	result = Hlt();

	if(gIsNT && mask)
	{
		SetThreadAffinityMask(GetCurrentThread(), mask);
	}

	return result;
}

//-----------------------------------------------------------------------------
//
// IO
//
//-----------------------------------------------------------------------------

DWORD WINAPI ReadIoPortDword(WORD port)
{
	if(gHandle == INVALID_HANDLE_VALUE){return 0;}

	DWORD	returnedLength = 0;
	BOOL	result = FALSE;
	DWORD	value = port;

	result = DeviceIoControl(
						gHandle,
						IOCTL_OLS_READ_IO_PORT_DWORD,
						&value,     
						sizeof(value),	// required 4 bytes
						&value,
						sizeof(value),
						&returnedLength,
						NULL
						);
	return value;
}

WORD WINAPI ReadIoPortWord(WORD port)
{
	if(gHandle == INVALID_HANDLE_VALUE){return 0;}

	DWORD	returnedLength = 0;
	BOOL	result = FALSE;
	WORD	value = 0;

	result = DeviceIoControl(
						gHandle,
						IOCTL_OLS_READ_IO_PORT_WORD,
						&port,     
						sizeof(port),
						&value,
						sizeof(value),
						&returnedLength,
						NULL
						);
	return value;
}

BYTE WINAPI ReadIoPortByte(WORD port)
{
	if(gHandle == INVALID_HANDLE_VALUE){return 0;}

	DWORD	returnedLength = 0;
	BOOL	result = FALSE;
	WORD	value = 0;

	result = DeviceIoControl(
						gHandle,
						IOCTL_OLS_READ_IO_PORT_BYTE,
						&port,
						sizeof(port),
						&value,
						sizeof(value),
						&returnedLength,
						NULL
						);
	return (BYTE)value;
}

VOID WINAPI WriteIoPortDword(WORD port, DWORD value)
{
	if(gHandle == INVALID_HANDLE_VALUE){return ;}

	DWORD	returnedLength = 0;
	BOOL	result = FALSE;
	DWORD   length = 0;
    OLS_WRITE_IO_PORT_INPUT inBuf;

	inBuf.LongData = value;
	inBuf.PortNumber = port;
	length = offsetof(OLS_WRITE_IO_PORT_INPUT, CharData) +
						 sizeof(inBuf.LongData);

	result = DeviceIoControl(
						gHandle,
						IOCTL_OLS_WRITE_IO_PORT_DWORD,
						&inBuf,
						length,
						NULL,
						0,
						&returnedLength,
						NULL
						);
}

VOID WINAPI WriteIoPortWord(WORD port, WORD value)
{
	if(gHandle == INVALID_HANDLE_VALUE){return ;}

	DWORD	returnedLength = 0;
	BOOL	result = FALSE;
	DWORD   length = 0;
    OLS_WRITE_IO_PORT_INPUT inBuf;

	inBuf.ShortData = value;
	inBuf.PortNumber = port;
	length = offsetof(OLS_WRITE_IO_PORT_INPUT, CharData) +
						 sizeof(inBuf.ShortData);

	result = DeviceIoControl(
						gHandle,
						IOCTL_OLS_WRITE_IO_PORT_WORD,
						&inBuf,
						length,
						NULL,
						0,
						&returnedLength,
						NULL
						);
}

VOID WINAPI WriteIoPortByte(WORD port, BYTE value)
{
	if(gHandle == INVALID_HANDLE_VALUE){return ;}

	DWORD	returnedLength = 0;
	BOOL	result = FALSE;
	DWORD   length = 0;
    OLS_WRITE_IO_PORT_INPUT inBuf;

	inBuf.CharData = value;
	inBuf.PortNumber = port;
	length = offsetof(OLS_WRITE_IO_PORT_INPUT, CharData) +
						 sizeof(inBuf.CharData);

	result = DeviceIoControl(
						gHandle,
						IOCTL_OLS_WRITE_IO_PORT_BYTE,
						&inBuf,
						length,
						NULL,
						0,
						&returnedLength,
						NULL
						);
}

//-----------------------------------------------------------------------------
//
// Physical Memory
//
//-----------------------------------------------------------------------------

DWORD WINAPI ReadPhysicalMemory(DWORD_PTR address, PBYTE buffer, DWORD count, DWORD unitSize)
{
	if(gHandle == INVALID_HANDLE_VALUE){return 0;}

	if(buffer == NULL)
	{
		return 0;
	}

	DWORD	returnedLength = 0;
	BOOL	result = FALSE;
	DWORD	size = 0;
	OLS_READ_MEMORY_INPUT inBuf;

	if(sizeof(DWORD_PTR) == 4)
	{
		inBuf.Address.HighPart = 0;
		inBuf.Address.LowPart = (DWORD)address;
	}
	else
	{
		inBuf.Address.QuadPart = address;
	}

	inBuf.UnitSize = unitSize;
	inBuf.Count = count;
	size = inBuf.UnitSize * inBuf.Count;

    result = DeviceIoControl(
                        gHandle,
                        IOCTL_OLS_READ_MEMORY,
                        &inBuf,
                        sizeof(OLS_READ_MEMORY_INPUT),
                        buffer,
                        size,
                        &returnedLength,
                        NULL
                        );

    if(result && returnedLength == size)
	{
		return count * unitSize;
	}
	else
	{
		return 0;
	}
}

DWORD WINAPI WritePhysicalMemory(DWORD_PTR address, PBYTE buffer, DWORD count, DWORD unitSize)
{
	if(gHandle == INVALID_HANDLE_VALUE){return 0;}

	if(buffer == NULL)
	{
		return 0;
	}

	DWORD	returnedLength = 0;
	BOOL	result = FALSE;
	DWORD	size = 0;
	DWORD	rCode = 0;
	OLS_WRITE_MEMORY_INPUT *inBuf;

	size = offsetof(OLS_WRITE_MEMORY_INPUT, Data) + count * unitSize;
	inBuf = (OLS_WRITE_MEMORY_INPUT*)malloc(size);
	if(sizeof(DWORD_PTR) == 4)
	{
		inBuf->Address.HighPart = 0;
		inBuf->Address.LowPart = (DWORD)address;
	}
	else
	{
		inBuf->Address.QuadPart = address;
	}
	inBuf->UnitSize = unitSize;
	inBuf->Count = count;
	memcpy(&inBuf->Data, buffer, count * unitSize);

    result = DeviceIoControl(
                        gHandle,
                        IOCTL_OLS_WRITE_MEMORY,
                        inBuf,     
                        size,
                        NULL,
                        0,
                        &returnedLength,
                        NULL
                        );
    if(result)
	{
		rCode = count * unitSize;
	}
	else
	{
		rCode = 0;
	}

	free(inBuf);

	return rCode;
}

//-----------------------------------------------------------------------------
//
// PCI Configuration Read
//
//-----------------------------------------------------------------------------

DWORD pciConfigRead(DWORD pciAddress, BYTE regAddress, PBYTE value, DWORD size)
{
	if(value == NULL)
	{
		return 0;
	}

	DWORD	returnedLength = 0;
	BOOL	result = FALSE;
	OLS_READ_PCI_CONFIG_INPUT inBuf;

	inBuf.PciAddress = pciAddress;
	inBuf.PciOffset = regAddress;

	result = DeviceIoControl(
                            gHandle,
                            IOCTL_OLS_READ_PCI_CONFIG,
                            &inBuf,
                            sizeof(inBuf),
                            value,
                            size,
                            &returnedLength,
                            NULL
                            );

	if(! result)
	{
		return GetLastError();
	}
        
	return 0;
}

BYTE WINAPI ReadPciConfigByte(DWORD pciAddress, BYTE regAddress)
{
	if(gHandle == INVALID_HANDLE_VALUE){return 0;}

	BYTE ret;
	if(pciConfigRead(pciAddress, regAddress, (BYTE*)&ret , sizeof(ret)))
	{
		return 0xffL;
	}
	return ret;
}

WORD WINAPI ReadPciConfigWord(DWORD pciAddress, BYTE regAddress)
{
	if(gHandle == INVALID_HANDLE_VALUE){return 0;}

	WORD ret;
	if((regAddress & 1) != 0)
	{
		return 0xFFFF;
	}

	if(pciConfigRead(pciAddress, regAddress, (BYTE*)&ret , sizeof(ret)))
	{
		return 0xFFFF;
	}
	return ret;
}

DWORD WINAPI ReadPciConfigDword(DWORD pciAddress, BYTE regAddress)
{
	if(gHandle == INVALID_HANDLE_VALUE){return 0;}

	DWORD ret;
	if((regAddress & 3) != 0)
	{
		return 0xFFFFFFFF;
	}

	if(pciConfigRead(pciAddress, regAddress, (BYTE*)&ret , sizeof(ret)))
	{
		return 0xFFFFFFFF;
	}
	return ret;
}

//-----------------------------------------------------------------------------
//
// PCI Configuration Write
//
//-----------------------------------------------------------------------------

DWORD pciConfigWrite(DWORD pciAddress, DWORD regAddress, PBYTE value, DWORD size)
{
	DWORD	returnedLength = 0;
	BOOL	result = FALSE;
	int		inputSize = 0;
	OLS_WRITE_PCI_CONFIG_INPUT	*inBuf;

	inputSize = offsetof(OLS_WRITE_PCI_CONFIG_INPUT, Data) + size;
	inBuf = (OLS_WRITE_PCI_CONFIG_INPUT*)malloc(inputSize);
	if(inBuf == NULL)
	{
		return 0xFFFFFFFF;
	}
	memcpy(inBuf->Data, value, size);
	inBuf->PciAddress = pciAddress;
	inBuf->PciOffset = regAddress;
	result = DeviceIoControl(
                            gHandle,
                            IOCTL_OLS_WRITE_PCI_CONFIG,
                            inBuf,
                            inputSize,
                            NULL,
                            0,
                            &returnedLength,
                            NULL
                            );

	free(inBuf);
	if(! result)
	{
		return GetLastError();
	}
	return 0;
}

VOID WINAPI WritePciConfigByte(DWORD pciAddress, BYTE regAddress, BYTE value)
{
	if(gHandle == INVALID_HANDLE_VALUE){return ;}

	pciConfigWrite(pciAddress, regAddress, (BYTE*)&value , sizeof(value));
}

VOID WINAPI WritePciConfigWord(DWORD pciAddress, BYTE regAddress, WORD value)
{
	if(gHandle == INVALID_HANDLE_VALUE){return ;}

	if((regAddress & 1) != 0)
	{
		return ;
	}
	pciConfigWrite(pciAddress, regAddress, (BYTE*)&value , sizeof(value));
}

VOID WINAPI WritePciConfigDword(DWORD pciAddress, BYTE regAddress, DWORD value)
{
	if(gHandle == INVALID_HANDLE_VALUE){return ;}

	if((regAddress & 3) != 0)
	{
		return ;
	}
	pciConfigWrite(pciAddress, regAddress, (BYTE*)&value , sizeof(value));
}

VOID WINAPI SetPciMaxBusIndex(BYTE max)
{
	if(gHandle == INVALID_HANDLE_VALUE){return ;}

	gPciMaxNumberOfBus = max;
}

DWORD WINAPI FindPciDeviceById(WORD vendorId, WORD deviceId, BYTE index)
{
	if(gHandle == INVALID_HANDLE_VALUE){return 0;}

	DWORD bus = 0, dev = 0, func = 0;
	DWORD count = 0;
	DWORD pciAddress = 0xFFFFFFFF;
	DWORD id = 0;
	DWORD ret = 0;	
	BOOL multiFuncFlag = FALSE;
	BYTE type = 0;
	count = 0;

	if(vendorId == 0xFFFF)
	{
		return 0xFFFFFFFF;
	}

	for(bus = 0; bus <= gPciMaxNumberOfBus; bus++)
	{
		for(dev = 0; dev < gPciNumberOfDevice; dev++)
		{
			multiFuncFlag = FALSE;
			for(func = 0; func < gPciNumberOfFunction; func++)
			{
				if(multiFuncFlag == 0 && func > 0)
				{
					break;
				}

				pciAddress = PciBusDevFunc(bus, dev, func);
				ret = pciConfigRead(pciAddress, 0, (BYTE*)&id, sizeof(id));
				if(ret == 0)
				{
					if(func == 0) // Is Multi Function Device
					{
						if(pciConfigRead(pciAddress, 0x0E, (BYTE*)&type, sizeof(type)) == 0)
						{
							if(type & 0x80)
							{
								multiFuncFlag = TRUE;
							}
						}
					}

					if(id == (vendorId | ((DWORD)deviceId << 16)))
					{
						if(count == index) 
						{
							return pciAddress;
						}
						count++;
						continue;
					}
				}
				else if(ret == OLS_ERROR_PCI_BUS_NOT_EXIST)
				{
					return 0xFFFFFFFF; 
				}
				else if(ret == OLS_ERROR_PCI_NO_DEVICE && func == 0)
				{
					break;
				}
			}
		}
	}
	return 0xFFFFFFFF;
}

DWORD WINAPI FindPciDeviceByClass(BYTE baseClass, BYTE subClass, BYTE programIf, BYTE index)
{
	if(gHandle == INVALID_HANDLE_VALUE){return 0;}

	DWORD bus = 0, dev = 0, func = 0;
	DWORD count = 0;
	DWORD pciAddress = 0xFFFFFFFF;
	DWORD conf[3] = {0};
	DWORD ret = 0;	
	BOOL multiFuncFlag = FALSE;
	BYTE type = 0;
	count = 0;

	for(bus = 0; bus <= gPciMaxNumberOfBus; bus++)
	{
		for(dev = 0; dev < gPciNumberOfDevice; dev++)
		{
			multiFuncFlag = FALSE;
			for(func = 0; func < gPciNumberOfFunction; func++)
			{
				if(multiFuncFlag == FALSE && func > 0)
				{
					break;
				}
				pciAddress = PciBusDevFunc(bus, dev, func);
				ret = pciConfigRead(pciAddress, 0, (BYTE*)conf , sizeof(conf));
				if(ret == 0)
				{
					if(func == 0) // Is Multi Function Device
					{ 
						if(pciConfigRead(pciAddress, 0x0E, (BYTE*)&type, sizeof(type)) == 0)
						{
							if(type & 0x80)
							{
								multiFuncFlag = TRUE;
							}
						}
					}
					if((conf[2] & 0xFFFFFF00) == 
							(((DWORD)baseClass << 24) |
							((DWORD)subClass << 16) |
							((DWORD)programIf << 8))
						)
					{
						if(count == index)
						{
							return pciAddress;
						}
						count++;
						continue;
					}
				}
				else if(ret == OLS_ERROR_PCI_BUS_NOT_EXIST)
				{
					return 0xFFFFFFFF;
				}
				else if(ret == OLS_ERROR_PCI_NO_DEVICE && func == 0)
				{
					break;
				}
			}
		}
	}
	return 0xFFFFFFFF;
}

